
#ifndef ISOSURFACE_H
#define ISOSURFACE_H

#include "Point3D.h"
#include "Vector3.h"
#include "Vector.h"


class Octree; // forward declaration

class OctreeNode {
public:
  OctreeNode(const unsigned char *vol, int _x, int _y, int _z, int _sx, int _sy, int _sz,
             int sizeX, int sizeY, int sizeZ, int depth, int maxDepth);
  ~OctreeNode();

  void marchingTets(const unsigned char *vol, unsigned char isovalue, Vector<Point3D> &verts,
                    Vector<Vector3> &normals, int sizeX, int sizeY, int sizeZ, int maxTris);

  friend class Octree;
private:
  int sx, sy, sz;
  int x, y, z;
  unsigned char min, max; // min and max isovalues in node
  OctreeNode *children[8];
}; // OctreeNode


class Octree {
public:
  Octree(): root(0) { }
  ~Octree() { if ( root ) delete root; }

  void build(const unsigned char *v, int sizeX, int sizeY, int sizeZ);
  void marchingTets(unsigned char isovalue, Vector<Point3D> &verts, Vector<Vector3> &normals, int maxTris);
private:
  OctreeNode *root;
  const unsigned char *vol; // volume data
  int sizeX, sizeY, sizeZ;  // dimensions of volume
}; // Octree


class Isosurface {
public:
  Isosurface(): verts(1000000), normals(1000000), vol(0) { }
  ~Isosurface() { }

  bool marchingTets(unsigned char isovalue, int maxTris);

  void clear();
  void render() const;
  void setVol(const unsigned char *v, int sx, int sy, int sz);

  static void marchTet(Vector<Point3D> &verts, Vector<Vector3> &normals, const unsigned char *vol, unsigned char isovalue, int x, int y, int z, int sizeX, int sizeY, int sizeZ);
private:
  Vector<Point3D> verts;
  Vector<Vector3> normals;
  float scaleX, scaleY, scaleZ;
  float dimX, dimY, dimZ;

  const unsigned char *vol;
  int sizeX, sizeY, sizeZ;

  Octree octree;
}; // Isosurface


#endif

