
#ifndef POINT3D_H
#define POINT3D_H

#include <cmath>
#include "Vector3.h"

class Point3D {
public:
   // components are stored in this union so they can be accessed as an array,
   // or by component name (x, y, z).
   union {
      float m[3];
      struct { float x, y, z; };
   }; // union

   Point3D() { }
   Point3D(float _x, float _y, float _z): x(_x), y(_y), z(_z) { }

   // overloaded operators
   inline float & operator [] (int i);
   inline Point3D operator + (const Vector3 &v) const;
   inline Point3D operator - (const Vector3 &v) const;
   inline const Point3D & operator += (const Vector3 &v);
   inline const Point3D & operator -= (const Vector3 &v);
   inline Vector3 operator - (const Point3D &p) const;            
   inline Point3D operator + (const Point3D &p) const;

   // functions
   inline void set(float _x, float _y, float _z);   
};

inline float & Point3D::operator [] (int i) {
   return m[i];
} // Point3D::operator []

// adds a vector to point, returns a new point
inline Point3D Point3D::operator + (const Vector3 &v) const {
   return Point3D(x + v.x, y + v.y, z + v.z);
} // Point3D::operator +

// subtracts a vector from a point and returns a new point
inline Point3D Point3D::operator - (const Vector3 &v) const {
  return Point3D(x - v.x, y - v.y, z - v.z);
} // Point3D::operator -

inline Point3D Point3D::operator + (const Point3D &p) const {
  return Point3D(x + p.x, y + p.y, z + p.z);
} // Point3D::operator +

// adds a vector to point and stores result in self
inline const Point3D & Point3D::operator += (const Vector3 &v) {
   x += v.x;
   y += v.y;
   z += v.z;

   return (*this);
} // Point3D::operator +=

// subtracts a vector from point and stores result in self
inline const Point3D & Point3D::operator -=(const Vector3 &v) {
  x -= v.x;
  y -= v.y;
  z -= v.z;

  return (*this);
} // Point3D::operator -=

// subtracts two points and returns resulting vector
inline Vector3 Point3D::operator - (const Point3D &p) const {
   return Vector3(x - p.x, y - p.y, z - p.z);
} // Point3D::operator -


// assign the values of the point's x, y, and z components to _x, _y, and _z respectively.
inline void Point3D::set(float _x, float _y, float _z) {
   x = _x;
   y = _y;
   z = _z;
} // Point3D::set()

#endif

