
#ifndef VECTOR_H
#define VECTOR_H

#include <cstdlib>
#include <cstring>

#define VECTOR_DEFAULT_INCREMENT 100

/////////////////////

template<class T>
class Vector {
public:
   Vector();
   Vector(int incVal);
   ~Vector();

   inline bool isFull() const { return full; }

   inline void insert(const T & el); // like STL vector's push_back()
   inline T& operator [](int i) const;
   const T *getData() const { return data; }
   int getSize() const { return count; }
   void removeLast() { if (count > 0) count--; } // like STL vector's pop_back (doesn't reduce amount 
                                                 //   of memory allocated)
   void clear() { count = 0; full = false; }
   void deallocate();
   void setData(int i, T value) { data[i] = value; }

private:
   inline void resize();
   int size;
   int count;
   int resizeInc;
   bool full;
   T *data;
}; // Vector

///////////////////////

// initialize vector with default increment
template<class T>
Vector<T>::Vector(): size(VECTOR_DEFAULT_INCREMENT), count(0),
          resizeInc(VECTOR_DEFAULT_INCREMENT), full(false), data(NULL) {
   data = (T *)malloc(sizeof(T)*VECTOR_DEFAULT_INCREMENT);

   if ( !data )
     full = true;
} // Vector::Vector()

///////////////////////

// initialize vector with specified increment
template<class T>
Vector<T>::Vector(int inc): size(inc), count(0),
          resizeInc(inc), full(false), data(NULL) {
   data = (T *)malloc(sizeof(T)*inc);

   if ( !data )
     full = true;
} // Vector::Vector()

/////////////////////

// just free up the data
template<class T>
Vector<T>::~Vector() {
   if (data)
     free(data);
} // Vector::~Vector()

/////////////////////

// simply insert an element at the end of the vector
// (equivalent of STL vector's push_back)
template<class T>
inline void Vector<T>::insert(const T &el) {
  if ( full )
    return;
 
  data[count++] = el;

  if ( count == size )
     resize();
} // Vector::insert()

/////////////////////

template<class T>
inline void Vector<T>::deallocate() {
  if ( data )
    free(data);
  data = (T *)malloc(sizeof(T)*resizeInc);

  if ( !data ) {
    count = 0;
    size = 0;
    full = true;
  }
  else {
    size = resizeInc;
    count = 0;
    full = false;
  }
} // Vector::deallocate()

/////////////////////

// should only be called by insert
template<class T>
inline void Vector<T>::resize() {
   T *newData = (T *)malloc(sizeof(T)*(size + resizeInc));

   if ( !newData ) {
      full = true;
      count = count > 0 ? count : 0;
   }
   else {
     memcpy(newData, data, sizeof(T)*size);
     free(data);
     data = newData;
     size += resizeInc;
   }
} // Vector::resize()

////////////////////

// overloaded [] operator
template<class T>
inline T& Vector<T>::operator[](int i) const {
   return data[i];
} // Vector::operator[]

/////////////////////

#endif

