
#ifndef VISCANVAS_H
#define VISCANVAS_H

#include <QWidget>
#include <QtOpenGL>
#include "trackball.h"
#include "vol.h"
#include "VolumeScene.h"


class VisCanvas : public QGLWidget {
Q_OBJECT

public:
  VisCanvas(QWidget *parent=0);
  virtual ~VisCanvas();

  bool loadVolume(const char *filename);
public slots:
  void buildIsoSlot();
  void isovalueChangedSlot(int value);
  void maxTriSlot(const QString &s);

private:
  // event handlers
  virtual void mousePressEvent(QMouseEvent *ev);
  virtual void mouseReleaseEvent(QMouseEvent *ev);
  virtual void mouseMoveEvent(QMouseEvent *ev);
  virtual void wheelEvent(QWheelEvent *ev);
  virtual void keyPressEvent(QKeyEvent *ev);

  virtual void initializeGL();
  virtual void resizeGL(int w, int h);
  virtual void paintGL();

  void render();

  enum RENDER_MODE { RENDER_SLICE, RENDER_ISOSURFACE, RENDER_VOLUME };

  RENDER_MODE renderMode;
  float zoom;              // z distance to translate camera
  Trackball trackball;
  Vol *vol;
  unsigned char isovalue;
  int maxTris;
  float stepSize;
  VolumeScene volumeScene;

  bool perspProj;

  // frustum settings
  float vFOV;                                // vertical field of view
  float left, right, top, bottom, nearZ, farZ; // clipping planes
}; // VisCanvas


#endif

