
#include <QMessageBox>
#include "VisWindow.h"


VisWindow::VisWindow() {
  setupUi(this);

  // create opengl display window
  visCanvas = new VisCanvas(canvasParent);
  visCanvas->resize( canvasParent->width(), canvasParent->height() );

  ///// setup signals/slots /////

  // menu
  connect( actionOpen, SIGNAL(triggered()), this, SLOT(openSlot()) );
  connect( actionQuit, SIGNAL(triggered()), this, SLOT(quitSlot()) );

  connect( horizontalSlider, SIGNAL(valueChanged(int)), visCanvas, SLOT(isovalueChangedSlot(int)) );

  // isosurface tab signals/slots
  connect( buildIsoButton, SIGNAL(pressed()), visCanvas, SLOT(buildIsoSlot()) );
  connect( maxTriEdit, SIGNAL(textChanged(const QString &)), visCanvas, SLOT(maxTriSlot(const QString &)) );
  
  // set default maximum # of triangles
  maxTriEdit->setText("5000000");
} // VisWindow::VisWindow()


// slot for "open" menu item
void VisWindow::openSlot() {
  QString s = QFileDialog::getOpenFileName(this, "", "", "vol files (*.vol)");

  // did the user select a file?
  if ( !s.isNull() )
    // attempt to load volume
    if ( !visCanvas->loadVolume(s.toAscii()) )
      QMessageBox::warning(this, "Error", "Unable to load file");
} // VisWindow::openSlot()


// slot for "quit" menu item
void VisWindow::quitSlot() {
  close();
} // VisWindow::quitSlot()


// called when user resizes window
void VisWindow::resizeEvent(QResizeEvent *ev) {
  splitter->resize( centralwidget->size() );
  
  if ( splitter->height() > 100 ) {
    int top = splitter->height() - 100;
    QList<int> sizeList;
    sizeList.insert(0, splitter->height()-top);
    sizeList.insert(0, top);
    splitter->setSizes(sizeList);
  }
  
  visCanvas->resize( canvasParent->width(), splitter->height() );
} // VisWindow::resizeEvent()


