
#ifdef _WIN32
   #include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include <cstring>

#include "VolumeScene.h"
#include "Vector3.h"
#include "Point3D.h"


VolumeScene::VolumeScene() {
  viewportWidth  = 512;
  viewportHeight = 512;

  sizeX = sizeY = sizeZ = 0;
  scaleX = scaleY = scaleZ = 0.0f;
  volData = 0;

  // default options
  maxTris = 0;
} // VolumeScene::initGL()


VolumeScene::~VolumeScene() {

} // VolumeScene::~VolumeScene()


// called when viewport is resized
bool VolumeScene::resize(int w, int h) {
  if ( w <= 0 || h <= 0 )
    return false;

  // update width and height
  viewportWidth  = w;
  viewportHeight = h;

  return true; // success
} // VolumeScene::resize()


void VolumeScene::setVol(const unsigned char *vol, int sx, int sy, int sz) {
  if ( sx <= 0 || sy <= 0 || sz <= 0 || !vol )
    return;

  volData = vol;
  sizeX = sx;
  sizeY = sy;
  sizeZ = sz;

  int l = sx > sy ? sx : sy;
  if ( sz > l )
    l = sz;

  longest = (float)l;
  scaleX = ((float)sx)/longest;
  scaleY = ((float)sy)/longest;
  scaleZ = ((float)sz)/longest;

  isosurface.clear(); // get rid of mesh
  isosurface.setVol(volData, sizeX, sizeY, sizeZ);
} // VolumeScene::setVol()


// invokes isosurface generation algorithm on volume
bool VolumeScene::buildIso(unsigned char isovalue, int maxTris) {
  return isosurface.marchingTets(isovalue, maxTris);
} // VolumeScene::buildIso()


void VolumeScene::display() {
  renderIso();
} // VolumeScene::display()


// renders isosurface using raycasting
void VolumeScene::renderIso() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  isosurface.render();
} // VolumeScene::renderIso()

