
#ifndef TRACKBALL_H
#define TRACKBALL_H

#include <cmath>
#include <Vector3.h>

#define PI 3.14159265f

/////////////////////////////

// a quaternion
class Quat {
public:
   float x, y, z, w;

   Quat(): x(0.0f), y(0.0f), z(0.0f), w(1.0f) { }
   Quat(float _x, float _y, float _z, float _w): 
      x(_x), y(_y), z(_z), w(_w) { }   
   Quat operator *(const Quat &q) const;
   const Quat & operator *= (const Quat &q);

   Quat conjugate() const { return Quat(-x, -y, -z, w); }
   void buildFromAxisAngle(const Vector3 &v, float angle);
   void toMatrix(float mat[]);
   float length() const { return sqrtf(x*x + y*y + z*z + w*w); }
   void normalize() { 
      float len = length();
      x /= len;
      y /= len;
      z /= len;
      w /= len;
   } // normalize()
}; // Quat

/////////////////////////////

class Trackball {
public:
   Trackball(): tracking(false), lastPos(0.0f, 0.0f, 0.0f) { }
   ~Trackball() { }

   void move(int x, int y, int w, int h);
   void look(float *mat);   
   void startMotion(int x, int y, int w, int h);

   inline void startTracking() { tracking = true; }
   inline void stopTracking() { tracking = false; }
   inline bool isTracking() const { return tracking; }
private:
   void pToV(int x, int y, int w, int h, Vector3 &p);

   bool tracking;
   Vector3 lastPos;
   Quat q;   
}; // Trackball

/////////////////////////////

#endif
